package controllers

import (
	"encoding/json"
	"fmt"
	"slices"

	oclib "cloud.o-forge.io/core/oc-lib"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about workspace
type SharedWorkspaceController struct {
	beego.Controller
}

// @Title Search
// @Description search shared workspace
// @Param	search		path 	string	true		"the word search you want to get"
// @Success 200 {shared workspace} models.shared_workspace
// @router /search/:search [get]
func (o *SharedWorkspaceController) Search() {
	// store and return Id or post with UUID
	search := o.Ctx.Input.Param(":search")
	o.Data["json"] = oclib.Search(nil, search, oclib.LibDataEnum(oclib.SHARED_WORKSPACE))
	o.ServeJSON()
}

// @Title Update
// @Description create shared workspaces
// @Param	id		path 	string	true		"the shared workspace id you want to get"
// @Param	body		body 	models.workspace	true		"The shared workspace content"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id [put]
func (o *SharedWorkspaceController) Put() {
	// store and return Id or post with UUID
	var paths = map[string]map[tools.METHOD]string{ // paths used to send to peers
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	fmt.Println("UPDATE", res)
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), res, id, caller)
	o.ServeJSON()
}

// @Title Create
// @Description create shared workspace
// @Param	data		body 	json	true		"body for data content (Json format)"
// @Success 200 {shared workspace} models.shared_workspace
// @router / [post]
func (o *SharedWorkspaceController) Post() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)

	o.Data["json"] = oclib.StoreOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), res, caller)
	o.ServeJSON()
}

// @Title GetAll
// @Description find shared workspace by id
// @Success 200 {shared_workspace} models.shared_workspace
// @router / [get]
func (o *SharedWorkspaceController) GetAll() {
	o.Data["json"] = oclib.LoadAll(oclib.LibDataEnum(oclib.SHARED_WORKSPACE))
	o.ServeJSON()
}

// @Title Get
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id [get]
func (o *SharedWorkspaceController) Get() {
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	o.ServeJSON()
}

// @Title Add Workspace
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workspace/:id2 [delete]
func (o *SharedWorkspaceController) RemoveWorkspace() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	newWorkspace := []string{}
	if slices.Contains(shared.Workspaces, id2) {
		for _, w := range shared.Workspaces {
			if w != id2 {
				newWorkspace = append(newWorkspace, w)
			}
		}
		shared.Workspaces = newWorkspace
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Workflow
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workflow/:id2 [delete]
func (o *SharedWorkspaceController) RemoveWorkflow() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	fmt.Println("RemoveWorkflow", r)
	shared := r.ToSharedWorkspace()
	newWorkflows := []string{}
	if slices.Contains(shared.Workflows, id2) {
		for _, w := range shared.Workflows {
			if w != id2 {
				newWorkflows = append(newWorkflows, w)
			}
		}
		shared.Workflows = newWorkflows
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Peer
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/peer/:id2 [delete]
func (o *SharedWorkspaceController) RemovePeer() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	newPeers := []string{}
	if shared.CreatorID != id2 {
		o.Data["json"] = map[string]interface{}{
			"data":  nil,
			"code":  409,
			"error": "You can't remove the creator from the shared workspace",
		}
		o.ServeJSON()
		return
	}
	if slices.Contains(shared.Peers, id) && shared.CreatorID != id2 {
		for _, peer := range shared.Peers {
			if peer != id2 {
				newPeers = append(newPeers, peer)
			}
		}
		shared.Peers = newPeers
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Rule
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/rule/:id2 [delete]
func (o *SharedWorkspaceController) RemoveRule() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	newRules := []string{}
	if shared != nil && slices.Contains(shared.Rules, id2) {
		for _, rule := range shared.Rules {
			if rule != id2 {
				newRules = append(newRules, rule)
			}
		}
		shared.Rules = newRules
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Workspace
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workspace/:id2 [post]
func (o *SharedWorkspaceController) AddWorkspace() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	if r.Code != 200 {
		o.Data["json"] = r
		o.ServeJSON()
		return
	}
	shared := r.ToSharedWorkspace()
	if shared != nil && !slices.Contains(shared.Workspaces, id2) {
		shared.Workspaces = append(shared.Workspaces, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Workflow
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workflow/:id2 [post]
func (o *SharedWorkspaceController) AddWorkflow() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	fmt.Println("AddWorkflow", id, id2)
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	if shared != nil && !slices.Contains(shared.Workflows, id2) {
		shared.Workflows = append(shared.Workflows, id2)
	}
	fmt.Println("AddWorkflow", shared.Workflows)

	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Peer
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/peer/:id2 [post]
func (o *SharedWorkspaceController) AddPeer() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to call other OC services
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	if shared != nil && !slices.Contains(shared.Peers, id2) {
		shared.Peers = append(shared.Peers, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Rule
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/rule/:id2 [post]
func (o *SharedWorkspaceController) AddRule() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to send to peers
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id)
	shared := r.ToSharedWorkspace()
	if shared != nil && !slices.Contains(shared.Rules, id2) {
		shared.Rules = append(shared.Rules, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the shared workspace
// @Param	id		path 	string	true		"The id you want to delete"
// @Success 200 {shared workspace} delete success!
// @router /:id [delete]
func (o *SharedWorkspaceController) Delete() {
	var paths = map[string]map[tools.METHOD]string{ // paths used to send to peers
		utils.SHARED_WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.SHARED_WORKSPACE)) + "/oc/shared/workspace/",
		},
		utils.WORKSPACE.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKSPACE)) + "/oc/workspace/",
		},
		utils.WORKFLOW.String(): {
			tools.DELETE: oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/:id",
			tools.POST:   oclib.GetPath(oclib.LibDataEnum(oclib.WORKFLOW)) + "/oc/workflow/",
		},
		utils.PEER.String(): {
			tools.POST: oclib.GetPath(oclib.LibDataEnum(oclib.PEER)) + "/oc/peer",
		},
	}
	id := o.Ctx.Input.Param(":id")
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	o.Data["json"] = oclib.DeleteOne(oclib.LibDataEnum(oclib.SHARED_WORKSPACE), id, caller)
	o.ServeJSON()
}
